@@ -3,16 +3,23 @@ selection <- function(selection, outcome,
                       weights = NULL,
                       subset,
                       method="ml",
+                      type = NULL,
                       start=NULL,
+                      boundaries = NULL,
                       ys=FALSE, xs=FALSE,
                       yo=FALSE, xo=FALSE,
                       mfs=FALSE, mfo=FALSE,
-                      print.level=0,
+                      printLevel=print.level, print.level=0,
                       ...) {
    ## Heckman-style sample-selection models
    ## selection:   formula
    ##              LHS: must be convertable to two-level factor (e.g. 0-1, 1-2, "A"-"B")
    ##              RHS: ordinary formula as in lm()
+   ## type         model type:
+   ##          2: tobit 2
+   ##          5: tobit 5
+   ##          treatment: spherical treatment model
+   ##          NULL: guess the correct type
    ## outcome:     formula, or a list of two formulas.
    ##              If the outcome contains one formula, it is
    ##              assumed that we observe the formula only if selection equals to the first level
@@ -23,9 +30,15 @@ selection <- function(selection, outcome,
    ##              the model frame of outcome and selection equation(s)
    ## First the consistency checks
    ## ...          additional arguments for tobit2fit and tobit5fit
-   ## 
-   type <- detectModelType(selection, outcome)
-   if(print.level > 0)
+   ##
+   if(is.null(type)) {
+      type <- detectModelType(selection, outcome)
+   }
+   checkModelType(type, selection, outcome)
+   if(length(outcome) == 1) {
+      outcome <- outcome[[1]]
+   }
+   if(printLevel > 0)
        cat("Tobit", type, "model\n")
    if(!missing(data)) {
       if(!inherits(data, "environment") & !inherits(data, "data.frame") & !inherits(data, "list")) {
@@ -48,14 +61,20 @@ selection <- function(selection, outcome,
    ## now check whether two-step method was requested
    cl <- match.call()
    if(method == "2step") {
-      if(type == 2)
+      if( !is.null( boundaries ) ) {
+         stop( "2-step estimation of a model with an outcome equation",
+            " that has an interval-variable as dependent variable",
+            " has not yet been implemented" )
+      }
+      if(type == 2) {
           twoStep <- heckit2fit(selection, outcome, data=data,
-            weights = weights, print.level = print.level, ... )
-      else if(type == 5)
+            weights = weights, printLevel = printLevel, ... )
+      } else if(type == 5) {
           twoStep <- heckit5fit(selection, outcome, data=data,
-            print.level = print.level, ... )
-      else
+            printLevel = printLevel, ... )
+      } else {
           stop("unknown type")
+      }
       twoStep$call <- cl
       class(twoStep) <- c("selection", class(twoStep))
       return(twoStep)
@@ -72,7 +91,7 @@ selection <- function(selection, outcome,
                                         # model.frame requires the parameter to
                                         # be 'formula'
    mfS <- eval(mfS, parent.frame())
-   mtS <- attr(mfS, "terms")
+   mtS <- terms(mfS)
    XS <- model.matrix(mtS, mfS)
    YS <- model.response(mfS)
    YSLevels <- levels( as.factor( YS ) )
@@ -92,20 +111,22 @@ selection <- function(selection, outcome,
    ## functions here.  Find bad rows and remove them later.
    ## We check XS and YS separately, because mfS may be a data frame with complex structure (e.g.
    ## including matrices)
-   badRow <- is.na(YS)
-   badRow <- badRow | apply(XS, 1, function(v) any(is.na(v)))
+   badRow <- is.na(YS) | is.infinite(YS)
+   badRow <- badRow | apply(XS, 1, function(v)
+      any(is.na(v) | is.infinite(v)))
    ## YO (outcome equation)
    ## Here we should include a possibility for the user to
    ## specify the model.  Currently just a guess.
-   binaryOutcome <- FALSE
-   if(type == 2) {
+   if(type %in% c(2, "treatment")) {
       oArg <- match("outcome", names(mf), 0)
                                         # find the outcome argument
       m <- match(c("outcome", "data", "subset",
                    "offset"), names(mf), 0)
       ## replace the outcome list by the first equation and evaluate it
       mfO <- mf[c(1, m)]
-      mfO$drop.unused.levels <- TRUE
+      if( is.null( boundaries ) ) {
+         mfO$drop.unused.levels <- TRUE
+      }
       mfO$na.action <- na.pass
       mfO[[1]] <- as.name("model.frame")
                                         # eval it as model frame
@@ -118,13 +139,32 @@ selection <- function(selection, outcome,
       mtO <- attr(mfO, "terms")
       XO <- model.matrix(mtO, mfO)
       YO <- model.response(mfO)
-      if(is.logical(YO) |
-         (is.factor(YO) & length(levels(YO)) == 2)) {
-         binaryOutcome <- TRUE
+      if( !is.null( boundaries ) ) {
+         outcomeVar <- "interval"
+      } else if(is.logical(YO) |
+            (is.factor(YO) & length(levels(YO)) == 2)) {
+         outcomeVar <- "binary"
+      } else {
+         outcomeVar <- "continuous"
+      }
+      if(type == 2) {
+         badRow <- badRow | ((is.na(YO) | is.infinite(YO))
+                             & (!is.na(YS) & YS == 1))
+         badRow <- badRow | (apply(XO, 1, function(v)
+            any(is.na(v) | is.infinite(v))) & (!is.na(YS) & YS == 1))
+                           # rows in outcome, which contain NA and are observable -> bad too
+                           # YO unobserved but should be observed
+                           # for tobit-2
+      } else if( type == "treatment" ) {
+         badRow <- badRow | is.na(YO) | is.infinite(YO)
+         badRow <- badRow | apply(XO, 1, function(v)
+            any(is.na(v) | is.infinite(v)))
+                           # YO unobserved in any case
+                           # for treatment models
+      } else {
+         stop( "Internal error ('badRow'). Please contact the maintainer",
+            " of this package" )
       }
-      badRow <- badRow | (is.na(YO) & (!is.na(YS) & YS == 1))
-      badRow <- badRow | (apply(XO, 1, function(v) any(is.na(v))) & (!is.na(YS) & YS == 1))
-                                        # rows in outcome, which contain NA and are observable -> bad too
 
       if( !is.null( weights ) ) {
          if( length( weights ) != length( badRow ) ) {
@@ -134,12 +174,12 @@ selection <- function(selection, outcome,
          badRow <- badRow | is.na( weights )
       }   
       
-      if(print.level > 0) {
+      if(printLevel > 0) {
          cat(sum(badRow), "invalid observations\n")
       }
       if( method == "model.frame" ) {
          mf <- mfS
-         mf <- cbind( mf, mfO[ , ! names( mfO ) %in% names( mf ), drop = FALSE ] )
+         mf <- cbind( mf, mfO[ , !(names( mfO ) %in% names( mf )), drop = FALSE ])
          return( mf[ !badRow, ] )
       }
       if( length( YSLevels ) != 2 ) {
@@ -152,57 +192,108 @@ selection <- function(selection, outcome,
       XO <- XO[!badRow,, drop=FALSE]
       YO <- YO[!badRow]
       weightsNoNA <- weights[ !badRow ]
-      YO[ YS == 0 ] <- NA
-      XO[ YS == 0, ] <- NA
+      if(type == 2) {
+         YO[ YS == 0 ] <- NA
+         XO[ YS == 0, ] <- NA
+      }
       NXS <- ncol(XS)
       NXO <- ncol(XO)
       iGamma <- 1:NXS
       iBeta <- max(iGamma) + seq(length=NXO)
-      if(!binaryOutcome) {
+      if( outcomeVar %in% c( "continuous", "interval" ) ) {
          iSigma <- max(iBeta) + 1
          iRho <- max(iSigma) + 1
-      }
-      else
+      } else if( outcomeVar == "binary" ) {
           iRho <- max(iBeta) + 1
+      } else {
+         stop( "Internal error ('iRho'). Please contact the maintainer",
+            " of this package" )
+      }
       nParam <- iRho
       twoStep <- NULL
-      if(is.null(start)) {
+      if(is.null(start) & is.null( boundaries ) ) {
                            # start values by Heckman 2-step method
          start <- numeric(nParam)
-         twoStep <- heckit2fit(selection, outcome, data=data,
-            print.level = print.level, weights = weights )
+         if(type == 2) {
+            twoStep <- heckit2fit(selection, outcome, data=data,
+                                  printLevel = printLevel,
+                                  weights = weights )
+         } else if( type == "treatment" ) {
+            twoStep <- heckitTfit(selection, outcome, data=data,
+                                  printLevel = printLevel,
+                                  weights = weights )
+         } else {
+            stop( "Internal error ('start'). Please contact the maintainer",
+               " of this package" )
+         }
          coefs <- coef(twoStep, part="full")
          start[iGamma] <- coefs[twoStep$param$index$betaS]
-         if(!binaryOutcome) {
+         if( outcomeVar == "continuous" ) {
             start[iBeta] <- coefs[twoStep$param$index$betaO]
             start[iSigma] <- coefs[twoStep$param$index$sigma]
-         }
-         else
+         } else if( outcomeVar == "binary" ) {
              start[iBeta] <- coefs[twoStep$param$index$betaO]/coefs[twoStep$param$index$sigma]
+         } else {
+            stop( "Internal error ('start-2'). Please contact the maintainer",
+               " of this package" )
+         }
          start[iRho] <- coefs[twoStep$param$index$rho]
-         if(start[iRho] > 0.99)
+         if(start[iRho] > 0.99) {
              start[iRho] <- 0.99
-         else if(start[iRho] < -0.99)
+         } else if(start[iRho] < -0.99) {
              start[iRho] <- -0.99
+         }
       }
       if(is.null(names(start))) {
-         if(!binaryOutcome) {
+         # add names to start values if not present
+         if( outcomeVar == "continuous" ) {
             names(start) <- c(colnames(XS), colnames(XO), "sigma",
                               "rho")
-         }
-         else
+         } else if( outcomeVar == "binary" ) {
             names(start) <- c(colnames(XS), colnames(XO), 
                               "rho")
-      }                                        # add names to start values if not present
-      if(!binaryOutcome) {
-         estimation <- tobit2fit(YS, XS, YO, XO, start, weights = weightsNoNA,
-                                 print.level=print.level, ...)
-         iErrTerms <- c(sigma=iSigma, rho=iRho )
+         } else if( outcomeVar != "interval" ) {
+            stop( "Internal error ('names'). Please contact the maintainer",
+               " of this package" )
+         }
       }
-      else {
-         estimation <- tobit2Bfit(YS, XS, YO, XO, start, weights = weightsNoNA,
-                                 print.level=print.level, ...)
-         iErrTerms <- c(rho=iRho)
+      if(type == 2) {
+         if( outcomeVar == "continuous" ) {
+            estimation <- tobit2fit(YS, XS, YO, XO, start, weights = weightsNoNA,
+                                    printLevel=printLevel, ...)
+            iErrTerms <- c(sigma=iSigma, rho=iRho )
+         } else if( outcomeVar == "interval" ) {
+            estimation <- tobit2Intfit(YS, XS, YO, XO,
+               boundaries = boundaries, start = start,
+               weights = weightsNoNA,
+               printLevel = printLevel, ... )
+            if( isTRUE( cl$returnLogLikStart ) ) {
+               return( estimation )
+            }
+            start <- estimation$start
+            iErrTerms <- c( logSigma = iSigma, atanhRho = iRho,
+               sigma = iRho + 1, sigmaSq = iRho + 2, rho = iRho + 3 )
+         } else if( outcomeVar == "binary" ) {
+            estimation <- tobit2Bfit(YS, XS, YO, XO, start, weights = weightsNoNA,
+                                     printLevel=printLevel, ...)
+            iErrTerms <- c(rho=iRho)
+         } else {
+            stop( "Internal error ('est-1'). Please contact the maintainer",
+               " of this package" )
+         }
+      } else if( type == "treatment" ) {
+         if( outcomeVar == "continuous" ) {
+            estimation <- tobitTfit(YS, XS, YO, XO, start, weights = weightsNoNA,
+                                    printLevel=printLevel, ...)
+            iErrTerms <- c(sigma=iSigma, rho=iRho )
+         } else {
+            stop("treatment effect models are only implemented",
+               " for continuous dependent variables",
+               " but not for ", outcomeVar, " dependent variables" )
+         }
+      } else {
+         stop( "Internal error ('est-2'). Please contact the maintainer",
+            " of this package" )
       }
       param <- list(index=list(betaS=iGamma,
                     betaO=iBeta,
@@ -215,8 +306,12 @@ selection <- function(selection, outcome,
                     levels=YSLevels
                            # levels[1]: selection 1; levels[2]: selection 2
                     )
-   }
-   else if(type == 5) {
+   } else if(type == 5) {
+      if( !is.null( boundaries ) ) {
+         stop( "estimation of tobit-5 models with an outcome equation",
+            " that has an interval-variable as dependent variable",
+            " has not yet been implemented" )
+      }
       ## extract the outcome formulas.  Anyone able to explain why do we need to do the complicated stuff?
       oArg <- match("outcome", names(mf), 0)
                                         # find the outcome argument
@@ -260,6 +355,12 @@ selection <- function(selection, outcome,
       YO2 <- model.response(mf2, "numeric")
       badRow <- badRow | (is.na(YO2) & (!is.na(YS) & YS == 1))
       badRow <- badRow | (apply(XO2, 1, function(v) any(is.na(v))) & (!is.na(YS) & YS == 1))
+      if( ( is.logical(YO1) | ( is.factor(YO1) & length(levels(YO1)) == 2 ) ) &
+            ( is.logical(YO2) | ( is.factor(YO2) & length(levels(YO2)) == 2  ) ) ){
+         outcomeVar <- "binary"
+      } else {
+         outcomeVar <- "continuous"
+      }
       if( method == "model.frame" ) {
          mf <- mfS
          mf <- cbind( mf, mf1[ , ! names( mf1 ) %in% names( mf ), drop = FALSE ] )
@@ -297,11 +398,11 @@ selection <- function(selection, outcome,
       twoStep <- NULL
       if(is.null(start)) {
          start <- numeric(nParam)
-         if(print.level > 0) {
+         if(printLevel > 0) {
             cat("Start values by Heckman 2-step method (", nParam, " componenets)\n", sep="")
          }
          twoStep <- heckit5fit(selection, as.formula(formula1), as.formula(formula2),
-                           data=data, print.level = print.level, ... )
+                           data=data, printLevel = printLevel, ... )
          ind <- twoStep$param$index
          start <- coef(twoStep, part="full")[c(ind$betaS,
                                   ind$betaO1, ind$sigma1, ind$rho1,
@@ -312,7 +413,7 @@ selection <- function(selection, outcome,
           names(start) <- c(colnames(XS), colnames(XO1), "sigma1", "rho1", colnames(XO2), "sigma2", "rho2")
                                         # add names to start values if not present
       estimation <- tobit5fit(YS, XS, YO1, XO1, YO2, XO2, start=start,
-                              print.level=print.level, ...)
+                              printLevel=printLevel, ...)
       param <- list(index=list(betaS=iBetaS,
                     betaO1=iBetaO1, sigma1=iSigma1, rho1=iRho1,
                     betaO2=iBetaO2, sigma2=iSigma2, rho2=iRho2,
@@ -326,6 +427,8 @@ selection <- function(selection, outcome,
                     levels=YSLevels
                            # levels[1]: selection 1; levels[2]: selection 2
                     )
+   } else {
+      stop("Unknown model type: '", type, "'")
    }
    ## now add the additional parameters into the resulting
    ## structure
@@ -335,21 +438,31 @@ selection <- function(selection, outcome,
                param=list(param),
                call=cl,
                termsS=mtS,
-               termsO=switch(as.character(type), "2"=mtO, "5"=list(mtO1, mtO2), "0"=NULL),
+               termsO=switch(as.character(type),
+                             "treatment"=, "2"=mtO,
+                             "5"=list(mtO1, mtO2),
+                             "0"=NULL),
                ys=switch(as.character(ys), "TRUE"=list(YS), "FALSE"=NULL),
                xs=switch(as.character(xs), "TRUE"=list(XS), "FALSE"=NULL),
                yo=switch(as.character(yo),
-               "TRUE"=switch(as.character(type), "2"=list(YO), "5"=list(YO1,
-                                                               YO2)), "FALSE"=NULL),
+                  "TRUE"=switch(as.character(type),
+                     "2"=, "treatment"=list(YO),
+                     "5"=list(YO1, YO2)),
+                  "FALSE"=NULL),
                xo=switch(as.character(xo),
-               "TRUE"=switch(as.character(type), "2"=list(XO), "5"=list(XO1, XO2)), "FALSE"=NULL),
+                  "TRUE"=switch(as.character(type),
+                     "2"=, "teatment"=list(XO),
+                     "5"=list(XO1, XO2)),
+                  "FALSE"=NULL),
                mfs=switch(as.character(mfs), "TRUE"=list(mfS[!badRow,]), "FALSE"=NULL),
-               mfo=switch(as.character(mfs),
-               "TRUE"=switch(as.character(type), "2"=list(mfO[!badRow,]),
-                  "5"=list(mf1[!badRow,], mf2[!badRow,]), "FALSE"=NULL))
+               mfo=switch(as.character(mfo),
+                  "TRUE"=switch(as.character(type),
+                     "2"=, "treatment"=list(mfO[!badRow,]),
+                     "5"=list(mf1[!badRow,], mf2[!badRow,]),
+                  "FALSE"=NULL))
                )
 
-   result$binaryOutcome <- binaryOutcome
+   result$outcomeVar <- outcomeVar
 
    class( result ) <- class( estimation ) 
    return(result)
